from pathlib import Path
from wizlib.parser import WizParser

from repozee.ai import AI
from repozee.command import RepoZeeCommand
from repozee.toolset import Toolset


class CommitCommand(RepoZeeCommand):

    name = 'commit'

    @classmethod
    def add_args(cls, parser: WizParser):
        super().add_args(parser)
        parser.add_argument('directory', default="", nargs='?')
        parser.add_argument('--yes', '-y', action='store_true')

    def handle_vals(self):
        super().handle_vals()
        if not self.provided('directory'):
            self.directory = str(Path.cwd())

    @RepoZeeCommand.wrap
    def execute(self):
        toolset = Toolset(self.directory)
        diff = toolset.git_diff()
        repozee = AI()
        response = repozee.ask('commit_from_diff', {'diff': diff})
        return response




from pathlib import Path
from gitignore_parser import parse_gitignore
import subprocess
from typing import Optional, Dict, Any
import yaml

from repozee import load_definition


class Toolset:

    definitions = yaml.safe_load(load_definition('tools.yml'))

    def __init__(self, directory):
        self.directory = Path(directory)

    def list(self) -> list[str]:
        result = []
        gitignore = None
        gitignore_file = self.directory / '.gitignore'
        if gitignore_file.exists():
            gitignore = parse_gitignore(gitignore_file)
        for file_path in self.directory.rglob('*'):
            if '.git' in file_path.parts:
                continue
            if gitignore and gitignore(str(file_path)):
                continue
            result.append(str(file_path.relative_to(self.directory)))
        return result

    def read_file(self, file_path: str) -> str:
        path = self.directory / file_path
        return path.read_text()

    def git_diff(self, path: Optional[str] = None) -> str:
        cmd = ['git', 'diff', 'HEAD']

        if path is not None:
            full_path = self.directory / path
            if not full_path.exists():
                raise FileNotFoundError(f"Path does not exist: {path}")
            cmd.append(str(full_path.relative_to(self.directory)))

        try:
            result = subprocess.run(
                cmd,
                capture_output=True,
                text=True,
                check=True,
                cwd=str(self.directory)
            )
            return result.stdout
        except subprocess.CalledProcessError as e:
            # Re-raise to allow caller to handle git errors
            raise



import os
from pathlib import Path
from datetime import date
import sys
import yaml

import anthropic
from jinja2 import Template

from repozee import load_definition


class AI:

    tools = yaml.safe_load(load_definition('tools.yml'))
    system_prompt = load_definition('system.md')
    user_prompt = Template(load_definition('user.md'))

    def __init__(self, toolset=None):
        self.toolset = toolset
        self.client = anthropic.Anthropic()

    def send(self, messages: list):
        params = {
            "model": "claude-3-5-sonnet-20241022",
            "max_tokens": 4096,
            "temperature": 0,
            "system": self.system_prompt,
            "messages": messages,
        }
        if self.toolset:
            params["tools"] = self.toolset.definitions
        message = self.client.messages.create(**params)
        return message

    def ask(self, template_name: str, data: dict) -> str:
        template = Template(load_definition(template_name + '.md'))
        messages = [
            {
                "role": "user",
                "content": [
                        {
                            "type": "text",
                            "text": template.render(
                                data=data,
                                today=date.today())
                        }
                ]
            }
        ]
        message = self.send(messages)
        response = next(c for c in message.content if c.type == 'text')
        return response.text

    def chat(self):
        messages = []
        while True:
            print('-'*80)
            if messages:
                question = input("\nYou: ").strip()
                print()
            else:
                question = "List"
            messages.append(
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": self.user_prompt.render(
                                question=question,
                                today=date.today())
                        }
                    ]
                }
            )
            message = self.send(messages)
            while messages[-1]['role'] != 'assistant':
                for content in message.content:
                    if content.type == 'text':
                        print(content.text)
                        print()
                        messages.append({
                            "role": "assistant",
                            "content": content.text
                        })
                    elif content.type == 'tool_use':
                        if content.name == 'quit':
                            return
                        else:
                            tool = getattr(self.toolset, content.name)
                            messages.append({
                                "role": "assistant",
                                "content": [dict(content)]
                            })
                            id = content.id
                            result = str(tool(**content.input))
                            messages.append({
                                "role": "user",
                                "content": [{
                                    "type": "tool_result",
                                    "tool_use_id": id,
                                    "content": result
                                }]
                            })
                            message = self.send(messages)



from pathlib import Path
import inspect

from wizlib.app import WizApp
from wizlib.ui_handler import UIHandler
from wizlib.config_handler import ConfigHandler

from repozee.command import RepoZeeCommand


def load_definition(filename: str) -> str:
    """
    Load a definition file relative to the calling module's definitions
    directory.

    Args:
        filename: Name of file to load (e.g. 'system.md', 'tools.yml')

    Returns:
        String content of the file

    Raises:
        FileNotFoundError: If definitions directory or file doesn't exist
    """

    caller_frame = inspect.currentframe().f_back
    caller_file = Path(inspect.getframeinfo(caller_frame).filename)

    definitions_dir = caller_file.parent / 'definitions'
    if not definitions_dir.exists():
        raise FileNotFoundError(
            f"No definitions directory found at {definitions_dir}")

    file_path = definitions_dir / filename
    if not file_path.exists():
        raise FileNotFoundError(f"Definition file not found: {file_path}")

    return file_path.read_text()


class RepoZeeApp(WizApp):

    base = RepoZeeCommand
    name = 'repozee'
    handlers = [UIHandler, ConfigHandler]

    # def __init__(self, **handlers):
    #     super().__init__(**handlers)
    #     self.storage = FileStorage(self.config.get('busy-storage-directory'))

    # def run(self, **vals):
    #     super().run(**vals)
